<?php
namespace some\name\space;

use my\other\{
    function f\foo,
    const c\foo,
    name\InterfaceB as InterfaceC,
    some\InterfaceD,
    const some\C
};

class ClassD extends foo implements InterfaceC {

}
